==> api-builder-plugin-fn-javascript-0.1.0.tgz: package/src/action.js <==
const vm = require('vm');

// filter out 'GLOBAL' and 'root' as these generate deprecation warnings
const safeContext = `
(function() {
	Function = undefined;
	const keys = Object.getOwnPropertyNames(this)
		.filter(a => a !== 'GLOBAL' && a !== 'root')
		.concat(['constructor']);
	for (const key of keys) {
		const item = this[key];
		if (item && typeof item.constructor === 'function') {
			this[key].constructor = undefined;
		}
	}
})();
`;

function execute(req, cb) {
	const { data, code, unsafe } = req.params;
	if (typeof code !== 'string') {
		return cb.error(null, 'invalid argument: code');
	}
	// determine the correct action to use
	const action = unsafe ? executeUnsafe : executeSafe;

	action(data, code).then(result => {
		cb.next(null, result);
	}, ex => {
		cb.error(null, formatException(ex));
	});
}

async function executeSafe(data, code) {
	// Define a new async function to resolve within a sandbox environment
	const script = new vm.Script(`(async function() { ${safeContext} ${code} })();`);
	return script.runInNewContext({ data });
}

async function executeUnsafe(data, code) {
	// Define a new async function to resolve without a sandbox environment
	// eslint-disable-next-line no-new-func
	const fn = new Function('data', 'require', `return (async function() { ${code} })();`);
	// to be functionally consistent with the safe version, make 'this' the global object
	// A number of items are not global - require, export, module, __dirname, __filename.
	// We just care about being able to use require in this function during unsafe mode.
	return fn.call(global, data, require);
}

function formatException(ex) {
	// The code provided by the user may not throw/reject with an Error object.
	// We would check ex instanceof Error but the different context of the vm causes this to fail
	if (ex instanceof Error || (typeof ex === 'object' && ex.stack && ex.message)) {
		// return a new object with the error message as an explicit property
		// JS Errors do not show the message when stringified.
		return {
			message: ex.message
		};
	}
	return ex;
}

module.exports = execute;

==> api-builder-plugin-fn-javascript-1.2.3.tgz: package/src/action.js <==
const vm = require('vm');

// filter out 'GLOBAL' and 'root' as these generate deprecation warnings
const safeContext = `
(function() {
	Function = undefined;
	const keys = Object.getOwnPropertyNames(this)
		.filter(a => a !== 'GLOBAL' && a !== 'root')
		.concat(['constructor']);
	for (const key of keys) {
		const item = this[key];
		if (key === 'data' && data && typeof item === 'object') {
			Object.defineProperty(this[key], 'constructor', {
				configurable: true
			});
		} else if (item && typeof item.constructor === 'function') {
			this[key].constructor = undefined;
		}
	}
})();
`;

function execute(req, cb) {
	const { data, code, unsafe } = req.params;
	if (typeof code !== 'string') {
		return cb.error(null, 'invalid argument: code');
	}
	// determine the correct action to use
	const action = unsafe ? executeUnsafe : executeSafe;

	action(data, code).then(result => {
		cb.next(null, result);
	}, ex => {
		cb.error(null, formatException(ex));
	});
}

async function executeSafe(data, code) {
	// Define a new async function to resolve within a sandbox environment
	const script = new vm.Script(`(async function() { ${safeContext} ${code} })();`);
	const value = script.runInNewContext({ data, console });
	// Since we deleted them in safeContext earlier,
	// restore the constructors of the external objects
	if (data && data.constructor === undefined) {
		// While it looks strange, deleting the instance property
		// allows the non-enumerable constructor on their prototype
		// to be accessible again
		delete data.constructor;
	}
	// eslint-disable-next-line no-console
	delete console.constructor;
	return value;
}

async function executeUnsafe(data, code) {
	// Define a new async function to resolve without a sandbox environment
	// eslint-disable-next-line no-new-func
	const fn = new Function('data', 'require', `return (async function() { ${code} })();`);
	// To be functionally consistent with the safe version, make 'this' the global object
	// A number of items are not global - require, export, module, __dirname, __filename.
	// We just care about being able to use require in this function during unsafe mode.
	return fn.call(global, data, require);
}

function formatException(ex) {
	// The code provided by the user may not throw/reject with an Error object.
	// We would check ex instanceof Error but the different context of the vm causes this to fail
	if (ex instanceof Error || (typeof ex === 'object' && ex.stack && ex.message)) {
		// return a new object with the error message as an explicit property
		// JS Errors do not show the message when stringified.
		return {
			message: ex.message
		};
	}
	return ex;
}

module.exports = execute;
